import _regeneratorRuntime from 'babel-runtime/regenerator';
import _slicedToArray from 'babel-runtime/helpers/slicedToArray';
import _asyncToGenerator from 'babel-runtime/helpers/asyncToGenerator';
import { createFilter } from 'rollup-pluginutils';
import { dirname, resolve } from 'path';
import { readFile } from 'fs';
import _Promise from 'babel-runtime/core-js/promise';

function promisify(nodeFn) {
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return new _Promise(function (resolve, reject) {
      nodeFn.apply(undefined, args.concat([function (error, result) {
        if (error) {
          reject(error);
        } else {
          resolve(result);
        }
      }]));
    });
  };
}

var readFileAsync = promisify(readFile);

// Borrowed from https://github.com/webpack/source-map-loader
var baseRegex = '\\s*[@#]\\s*sourceMappingURL\\s*=\\s*([^\\s]*)';
var regex1 = new RegExp('/\\*' + baseRegex + '\\s*\\*/'); // Matches /* ... */ comments
var regex2 = new RegExp('//' + baseRegex + '($|\n|\r\n?)'); // Matches // .... comments
var regexDataUrl = /data:(.*?)(?:;charset=(.*?))?(?:;(base64))?,(.+)/i; // Matches data urls

function sourceMapsPlugin() {
  var _ref = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  var include = _ref.include;
  var exclude = _ref.exclude;

  var filter = createFilter(include, exclude);

  return {
    load: function load(id) {
      var _this = this;

      return _asyncToGenerator(_regeneratorRuntime.mark(function _callee() {
        var code, match, _match, sourceMapComment, sourceMapURL, dataUrlMatch, rawMap, _dataUrlMatch, dataUrl, mimeType, _dataUrlMatch$, charset, _dataUrlMatch$2, encoding, data, sourceMapPath, map;

        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                if (filter(id)) {
                  _context.next = 2;
                  break;
                }

                return _context.abrupt('return', null);

              case 2:
                code = void 0;
                _context.prev = 3;
                _context.next = 6;
                return readFileAsync(id, 'utf8');

              case 6:
                code = _context.sent;
                _context.next = 12;
                break;

              case 9:
                _context.prev = 9;
                _context.t0 = _context['catch'](3);
                return _context.abrupt('return', null);

              case 12:

                // Check for source map comments
                match = regex1.exec(code) || regex2.exec(code);

                // No source map detected, return code

                if (!(match === null)) {
                  _context.next = 15;
                  break;
                }

                return _context.abrupt('return', code);

              case 15:
                _match = _slicedToArray(match, 2);
                sourceMapComment = _match[0];
                sourceMapURL = _match[1];

                // Remove the source map comment, and make sure we don't return an empty string

                code = code.replace(sourceMapComment, '') || '\n';

                // Check if the source map is inlined, i.e. it uses a data url
                dataUrlMatch = regexDataUrl.exec(sourceMapURL);
                rawMap = void 0;

                if (!(dataUrlMatch !== null)) {
                  _context.next = 40;
                  break;
                }

                /* eslint-disable no-unused-vars */
                _dataUrlMatch = _slicedToArray(dataUrlMatch, 5);
                dataUrl = _dataUrlMatch[0];
                mimeType = _dataUrlMatch[1];
                _dataUrlMatch$ = _dataUrlMatch[2];
                charset = _dataUrlMatch$ === undefined ? 'utf8' : _dataUrlMatch$;
                _dataUrlMatch$2 = _dataUrlMatch[3];
                encoding = _dataUrlMatch$2 === undefined ? 'base64' : _dataUrlMatch$2;
                data = _dataUrlMatch[4];
                /* eslint-enable */

                _context.prev = 30;

                rawMap = new Buffer(data, encoding).toString(charset);
                _context.next = 38;
                break;

              case 34:
                _context.prev = 34;
                _context.t1 = _context['catch'](30);

                console.error('rollup-plugin-sourcemaps: Failed parsing data url: ' + _context.t1);
                return _context.abrupt('return', code);

              case 38:
                _context.next = 51;
                break;

              case 40:
                sourceMapPath = resolve(dirname(id), sourceMapURL);
                _context.prev = 41;
                _context.next = 44;
                return readFileAsync(sourceMapPath, 'utf8');

              case 44:
                rawMap = _context.sent;
                _context.next = 51;
                break;

              case 47:
                _context.prev = 47;
                _context.t2 = _context['catch'](41);

                console.error('rollup-plugin-sourcemaps: Could not open source map: ' + _context.t2);
                return _context.abrupt('return', code);

              case 51:
                map = void 0;
                _context.prev = 52;

                map = JSON.parse(rawMap);
                _context.next = 60;
                break;

              case 56:
                _context.prev = 56;
                _context.t3 = _context['catch'](52);

                console.error('rollup-plugin-sourcemaps: Failed parsing raw source map from ' + id + ': ' + _context.t3);
                return _context.abrupt('return', code);

              case 60:
                return _context.abrupt('return', { code: code, map: map });

              case 61:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, _this, [[3, 9], [30, 34], [41, 47], [52, 56]]);
      }))();
    }
  };
}

export default sourceMapsPlugin;
//# sourceMappingURL=rollup-plugin-sourcemaps.esm.js.map